module correlation_mod
  ! Module to compute correlations 
  !
  ! Jason Chang
  ! January 2016

  !=========================================
  ! includes
  !=========================================
  use fourier1d_mod
  implicit none


  !=========================================
  ! declare module variables
  !=========================================
  integer                          :: n1

  contains


  !=========================================
  ! initialize correlation
  !=========================================
  subroutine correlation_init(n1_in)
    ! declare variables
    integer                        :: n1_in

    ! pass variables
    n1 = n1_in

    ! initialize fourier module
    call fft1d_init(n1_in)

  end subroutine


  !=========================================
  ! close correlation
  !=========================================
  subroutine correlation_close()
    ! close fourier module
    call fft1d_close()
  end subroutine


  !=========================================
  ! perform correlation
  !=========================================
  subroutine correlation(trace1,trace2,corr)
    ! declare variables
    real,dimension(:),intent(in)     :: trace1,trace2
    real,dimension(:),intent(out)    :: corr
    real,allocatable,dimension(:)    :: temp
    complex,allocatable,dimension(:) :: ctrace1,ctrace2
    complex,allocatable,dimension(:) :: ccorr

    ! allocate
    allocate(ctrace1(n1))
    allocate(ctrace2(n1))
    allocate(ccorr(n1))
    allocate(temp((n1+1)/2))

    ! convert traces to complex
    ctrace1 = cmplx(trace1,0.)
    ctrace2 = cmplx(trace2,0.)

    ! fourier transform inputs
    call fft1d(.false.,1.,ctrace1)
    call fft1d(.false.,1.,ctrace2)

    ! perform correlation
    ccorr = ctrace1*conjg(ctrace2)

    ! inverse fourier transform output
    call fft1d(.true.,1.,ccorr)

    ! convert output to real and shift
    corr = real(ccorr)
    temp(:) = corr(n1-(n1+1)/2+1:n1)
    corr(n1-(n1+1)/2+1:n1) = corr(1:(n1+1)/2)
    corr(1:(n1+1)/2) = temp(:)

    ! clean up
    deallocate(ctrace1,ctrace2,ccorr,temp)

  end subroutine

end module
