program jcEnvelope
  ! Program to calculate the envelope of a time series
  ! Modules needed: sep, fourier_mod
  !
  ! Inputs:
  !   1) history file of time series
  !
  ! Outputs:
  !   1) history file of envelope of time series
  !
  ! Jason Chang
  ! December 2014

  !================================
  ! standard declarations
  !================================
  use sep
  use fourier_mod
  implicit none


  !================================
  ! declare variables
  !================================
  ! program variables
  integer                             :: ii,jj

  ! input variables
  integer                             :: n1,n2,n3
  real                                :: o1,o2,o3,d1,d2,d3
  real,allocatable,dimension(:)       :: dat

  ! fourier variables
  integer                             :: nf
  real                                :: of,df
  complex,allocatable,dimension(:)    :: cdat


  !================================
  ! open I/O
  !================================
  call sep_init()

  ! input parameters
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('o1',o1,0.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  call from_history('d1',d1,1.)
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)


  !================================
  ! initialize
  !================================
  ! calculate fourier units
  nf = n1
  df = 1/((n1-1)*d1)
  of = -1./(2*d1)

  ! allocate
  allocate(dat (n1))
  allocate(cdat(nf))

  ! initialization
  call fft_init(n1)


  !================================
  ! kernel
  !================================
  do ii = 1,n2*n3
    ! read in data and change to complex
    call sreed('in',dat,4*n1)
    cdat = cmplx(dat,0.)

    ! fourier transform the signal
    call fft1d(.false.,1.,cdat)

    ! multiply by modified heaviside function
    do jj = 1,nf/2
      cdat(jj) = 0.
    end do
    do jj = nf/2+2,nf
      cdat(jj) = 2.*cdat(jj)
    end do

    ! inverse fourier transform
    call fft1d(.true.,1.,cdat)

    ! write out data
    call srite('out',abs(cdat),4*n1)

  end do


  !================================
  ! clean up
  !================================
  call sep_close()
  call fft_close()
  deallocate(dat,cdat)
  call exit(0)

end program
