program jcLMO_irreg
  ! Jason Chang
  ! May 2017
  !
  ! Linear moveout on 1D data
  ! Trace spacing is irregular
  !
  ! Input:
  !	- stdin: seismic gathers (n1:time, n2:space)
  !     - offset: offset history file
  !     - vel: moveout velocity
  !     - soffset: source offset (to account for active sources)
  !
  ! Output:
  !	- LMOd seismic gather


  !--------------------------------------------
  ! standard declarations
  !--------------------------------------------
  use sep
  implicit none


  !--------------------------------------------
  ! declare variables
  !--------------------------------------------
  integer                                :: ii,jj,it,itau
  integer                                :: nt,nx,n1h
  real                                   :: dt,ot,dx,ox
  real                                   :: vel,slow,tau,x,soffset
  real,allocatable,dimension (:)         :: time
  real,allocatable,dimension (:)         :: dat,dout


write(0,*) "here"
  !--------------------------------------------
  ! open I/O
  !--------------------------------------------
  call sep_init()

  ! history file variables
  call from_history('n1',nt,1 )
  call from_history('o1',ot,1.)
  call from_history('d1',dt,1.)
  call from_history('n2',nx,1 )
  call from_history('o2',ox,1.)
  call from_history('d2',dx,1.)

  ! lmo variables
  call from_param("vel",vel,1.)
  call from_param("soffset",soffset,0.)
  slow = 1./vel
  if (exist_file("offset")) then
    call from_aux("offset","n1",n1h)
    if (n1h.ne.nx) call seperr("Number of offsets do not match")
  else
    call seperr("Need offset history file")
  end if


  ! out to history file
  call to_history('n1',nt)
  call to_history('o1',ot)
  call to_history('d1',dt)
  call to_history('n2',nx)
  call to_history('o2',ox)
  call to_history('d2',dx)

  call sep_close()


  !--------------------------------------------
  ! allocate
  !--------------------------------------------
  allocate(dat(nt))
  allocate(dout(nt))
  allocate(time(nt))

  ! set up time vector outside of loop
  do ii=1,nt
    time(ii) = ot+(ii-1)*dt
  end do



  !--------------------------------------------
  ! kernel
  !--------------------------------------------
  do ii=1,nx
    ! read in data
    call sreed('in',dat,4*nt)
    call sreed('offset',x,4)
    dout = 0.

    ! moveout
    do jj=1,nt
      tau = time(jj)-(slow*abs(x-soffset))
      !if (time(jj).ge.0) then
      !  tau = time(jj)-(slow*abs(x-soffset))
      !else
      !  tau = time(jj)+(slow*abs(x-soffset))
      !end if
      itau = 1.5 + (tau-ot)/dt
      if (itau.ge.1 .and. itau.le.nt) then
        dout(itau) = dout(itau)+dat(jj)
      end if
    end do

    ! write out data
    call srite('out',dout,4*nt)
  end do


  !--------------------------------------------
  ! clean up
  !--------------------------------------------
  deallocate(dat,dout,time)

end program
