#########################################################
# Code to create spectragrams from file synchronized in time
#
# Jason Chang
# October 2014
# Update: July 2015 (updated for Forties)
# Update: March 2017 (updated for synchronized time windows)
##########################################################


# import modules
import sys
import commands
import math
import re
import os


# run parameters
run = True
SEPLIB = '/opt/SEP/SEP8.1/bin'


# parse command line
def parse_commandline():
  # check length of inputs
  if len(sys.argv) < 5:
    print "Usage:"
    print "	python %s [search directory] [station index] [component] [output file]"
    sys.exit()

  # grab information
  sdir = sys.argv[1]
  sid = sys.argv[2]
  comp = sys.argv[3]
  outfile = sys.argv[4]

  # clean up
  return sdir,sid,comp,outfile


# window all files and cat
def create_spectrogram(sdir,sid,comp,outfile):
  # regular expression
  look = re.compile("^p_APAC_01_%s_\d+\.H$" %(comp))

  # find all files in the given directory with the given format
  files = []
  for root,dirnames,filenames in os.walk(sdir):
    for filename in filenames:
      if look.search(filename):
        files.append(sdir+"/"+filename)
  files.sort()

  # look over all found files
  cfiles = []
  cc = 0
  for ff in files:
    # window the station id for each of those
    comm = """cp %s here.h; echo "hff=-1" >> here.h""" %(ff)
    print comm
    if run:
      commands.getoutput(comm)
    ofile = "temp%02d.h" %(cc)
    comm = """Window < here.h n2=1 f2=%s | ./Bin/jcRemoveMean.x | ./Bin/srTaper1d.x opt=7 | ./Bin/jcSpectra1d.x opt=2 > %s""" %(sid,ofile)
    print comm
    if run:
      commands.getoutput(comm)
    cfiles.append("%s" %(ofile))
    cc += 1

  # cat the windowed files together
  comm = """%s/Scat3d axis=2 %s > %s""" %(SEPLIB," ".join(cfiles),outfile)
  print comm
  if run:
    commands.getoutput(comm)


# grab information about the input trace
def create_spectrogramaaaa(infile,outfile):
  ### grab axes information ###
  get_n1 = """Get < %s n1 parform=n """ %(infile)
  print get_n1
  status,n1 = commands.getstatusoutput(get_n1)
  n1 = int(n1)
  get_d1 = """Get < %s d1 parform=n """ %(infile)
  print get_d1
  status,d1 = commands.getstatusoutput(get_d1)
  d1 = float(d1)
  get_n2 = """Get < %s n2 parform=n """ %(infile)
  print get_n2
  status,n2 = commands.getstatusoutput(get_n2)
  n2 = int(n2)

  ### grab start time information ###
  wind_com = """Window3d < %s n2=1 synch=1 > tmp.h""" %(infile)
  print wind_com
  commands.getstatusoutput(wind_com)
  # check the day
  disfil_com = """Dis3dhead < tmp.h key_list=DAY"""
  print disfil_com
  disfil = commands.getstatusoutput(disfil_com)
  day = int(disfil[1].split()[3])
  # check the hour
  disfil_com = """Dis3dhead < tmp.h key_list=HOUR"""
  print disfil_com
  disfil = commands.getstatusoutput(disfil_com)
  hour = int(disfil[1].split()[3])
  # check the minute
  disfil_com = """Dis3dhead < tmp.h key_list=MINUTE"""
  print disfil_com
  disfil = commands.getstatusoutput(disfil_com)
  min = int(disfil[1].split()[3])
  # make sure seconds is zero
  disfil_com = """Dis3dhead < tmp.h key_list=SECOND"""
  print disfil_com
  disfil = commands.getstatusoutput(disfil_com)
  sec = int(disfil[1].split()[3])
  if (sec!=0):
    print "Seconds are not starting at zero"
  # set starting time
  tsec = (day*24*60*60)+(hour*60*60)+(min*60)+sec
  o2   = float(tsec)/60./60./24.
  d2   = lwindow/60./60./24./2

  ### round up to nearest half-interval of window ###
  # starting seconds
  ssec  = min*60
  # nearest seconds
  nsec  = lwindow*0.5
  # round up to nearest nsec
  nwind = math.ceil(float(ssec)/nsec)
  rsec  = nwind*nsec
  skip  = rsec-ssec
  # convert to number of samples to skip
  nskip = skip/d1
  
  ### grab end time information ###
  # check the minute
  wind_com = """Window3d < %s n2=1 f2=%d synch=1 > tmp.h""" %(infile,n2-1)
  print wind_com
  commands.getstatusoutput(wind_com)
  disfil_com = """Dis3dhead < tmp.h key_list=MINUTE"""
  print disfil_com
  disfil = commands.getstatusoutput(disfil_com)
  min = int(disfil[1].split()[3])
  print min
  # make sure seconds is zero
  disfil_com = """Dis3dhead < tmp.h key_list=SECOND"""
  print disfil_com
  disfil = commands.getstatusoutput(disfil_com)
  sec = int(disfil[1].split()[3])
  if (sec!=0):
    print "Seconds are not starting at zero"

  ### round down to nearest half-interval of window ###
  # ending seconds (add one bc times in headers are for start)
  esec  = (min+1)*60
  # nearest seconds
  nsec  = lwindow*0.5
  # round down to nearest nsec
  nwind = math.floor(float(esec)/nsec)
  rsec  = nwind*nsec
  cut   = esec-rsec
  # convert to number of samples to cut
  ncut  = cut/d1
  print ncut,nskip

  ### reshape file and trim last minute off files ###
  cp_com = """cp %s hold1.h""" %(infile)
  print cp_com
  commands.getstatusoutput(cp_com)
  nn = n1*n2
  echo_com = """echo 'n1=%s n2=1 f2=0 d2=1 hff=-1' >> hold1.h""" %(nn)
  print echo_com
  commands.getstatusoutput(echo_com)

  ### apply starting index and number of samples ###
  nsamp = nn-nskip-ncut
  wind_com = """Window < hold1.h f1=%s n1=%s > hold2.h""" %(nskip,nsamp)
  print wind_com
  commands.getstatusoutput(wind_com)

  ### patch files ###
  nwind1  = int(lwindow/d1)
  npatch  = nsamp/nwind1
  npatch1 = (2*npatch)-1
  patch_com = """Patch < hold2.h npatch1=%d nwind1=%d > hold3.h""" %(npatch1,nwind1)
  print patch_com
  commands.getstatusoutput(patch_com)

  ### compute spectrum ###
  spec_com = """./Bin/jcRemoveMean.x < hold3.h | ./Bin/srTaper1d.x opt=5 | ./Bin/jcSpectra1d.x opt=2 > %s """ %(outfile)
  print spec_com
  commands.getstatusoutput(spec_com)

  ### echo in new coordinates ###
  echo_com = """echo 'o2=%s d2=%s' >> %s""" %(o2,d2,outfile)
  print echo_com
  commands.getstatusoutput(echo_com)



# reshape the trace
def create_spectrogram2(infile,outfile,n1,d1,n2,o2,d2):
  # figure out how many time samples to take in total
  nsec = n1*d1

  # determine number of 10-second segments to grab
  factor = lwindow/nsec
  n2new1 = int(float(n2)/factor)*factor

  # temp files
  temp1 = 'temp1.h'
  temp2 = 'temp2.h'

  # trim the file down to integer number of segments
  window1 = """Window < %s n2=%d > %s """ %(infile,n2new1,temp1)
  print window1
  if run:
    status,output = commands.getstatusoutput(window1)
    print output

  # echo into one long trace
  nsamp = n2new1*n1
  echo1 = """echo "n1=%d n2=1 d2=1 o2=0 hff=-1" >> %s""" %(nsamp,temp1)
  print echo1
  if run:
    status,output = commands.getstatusoutput(echo1)
    print output

  # patch
  nwind1  = int(lwindow/d1)
  n2new2 = nsamp/nwind1
  npatch1 = (2*n2new2)-1
  patch = """Patch < %s npatch1=%d nwind1=%d > %s """ %(temp1,npatch1,nwind1,temp2)
  print patch
  if run:
    status,output = commands.getstatusoutput(patch)
    print output


  # compute spectrum
  spectrum = """./Bin/jcRemoveMean.x < %s | ./Bin/srTaper1d.x opt=7 | ./Bin/jcSpectra1d.x opt=2 > %s """ %(temp2,outfile)
  print spectrum
  if run:
    status,output = commands.getstatusoutput(spectrum)
    print output

  # echo in new coordinates
  echo_com = """echo 'o2=%s d2=%s' >> %s""" %(o2,d2,outfile)
  print echo_com
  if run:
    commands.getstatusoutput(echo_com)


# run commands
sdir,sid,comp,outfile = parse_commandline()
create_spectrogram(sdir,sid,comp,outfile)
