# Python code for creating files of simulateneous recordings for Forties
#   - only takes in one file at a time
#   - note: issue here is determining the number of patches to have
#   - can potentially set second axis to have the time info (julian patch)
#
# Jason Chang
# July 2015

#-------------------------------
# import modules
#-------------------------------
import sys
import commands
import os
import re
import math


#-------------------------------
# parse command line
#-------------------------------
def parse_commandline():
  # check number of arguments
  if (len(sys.argv) < 4):
    print "Usage:"
    print "	python %s [filename] [patch length in seconds] [prefix]" %(sys.argv[0])
    sys.exit()

  # grab variables
  file     = sys.argv[1]
  patchsec = int(sys.argv[2])
  prefix   = sys.argv[3]

  # make sure file exists
  exist = os.path.exists(file)
  if (not exist):
    print "File does not exist"
    sys.exit()

  # clean up
  return file,patchsec,prefix


#-------------------------------
# find beginning and end times
#-------------------------------
def grab_time_stamps(file):
  # initialize arrays
  tdict = {}
  pdict = {}

  # grab the number of time segments and other information
  n1_com = """Get < %s n1 parform=n """ %(file)
  print n1_com
  n1 = int(commands.getoutput(n1_com))
  d1_com = """Get < %s d1 parform=n """ %(file)
  print d1_com
  d1 = float(commands.getoutput(d1_com))
  n2_com = """Get < %s n2 parform=n """ %(file)
  print n2_com
  n2 = int(commands.getoutput(n2_com))
  print "n2:",n2
  pdict['n1'] = n1
  pdict['d1'] = d1
  # calculate and grab beginning minutes of each trace
  wind_com = """Window3d < %s n2=1 synch=1 > tmp.h""" %(file)
  print wind_com
  commands.getoutput(wind_com)
  # day information
  disfil_com = """Dis3dhead < tmp.h key_list=DAY"""
  print disfil_com
  output = commands.getoutput(disfil_com)
  day = int(output.split()[3])
  # hour information
  disfil_com = """Dis3dhead < tmp.h key_list=HOUR"""
  print disfil_com
  output = commands.getoutput(disfil_com)
  hour = int(output.split()[3])
  # minute information
  disfil_com = """Dis3dhead < tmp.h key_list=MINUTE"""
  print disfil_com
  output = commands.getoutput(disfil_com)
  minute = int(output.split()[3])
  # starting time
  tstart = (day*24*60)+(hour*60)+(minute)
  print "tstart:",tstart
  # calculate and grab ending minutes of each trace
  wind_com = """Window3d < %s n2=1 f2=%d synch=1 > tmp.h""" %(file,n2-1)
  print wind_com
  commands.getoutput(wind_com)
  # day information
  disfil_com = """Dis3dhead < tmp.h key_list=DAY"""
  print disfil_com
  output = commands.getoutput(disfil_com)
  day = int(output.split()[3])
  # hour information
  disfil_com = """Dis3dhead < tmp.h key_list=HOUR"""
  print disfil_com
  output = commands.getoutput(disfil_com)
  hour = int(output.split()[3])
  # minute information
  disfil_com = """Dis3dhead < tmp.h key_list=MINUTE"""
  print disfil_com
  output = commands.getoutput(disfil_com)
  minute = int(output.split()[3])
  # ending time
  tend = (day*24*60)+(hour*60)+(minute)+1
  print "tend:",tend

  # place all file information into dictionary
  tdict['%s' %(file)] = (tstart,tend)

  # clean up
  return tdict,pdict


#-------------------------------
# pad and segment
#-------------------------------
def pad_patch(tdict,pdict,patchsec,prefix):
  # unpack info
  n1 = pdict['n1']
  d1 = pdict['d1']

  # convert patchsec to minutes
  mpatch = patchsec/60.
  mround = mpatch/2.

  # grab info from dictionary
  times = tdict.values()

  # grab start time and round to ceiling patch integer
  tt     = max([time[0] for time in times])
  ttmin  = math.ceil(float(tt)/mround)
  pstart = ttmin*mround
  # grab end time and round to floor patch integer
  tt    = min([time[1] for time in times])
  ttmax = math.floor(float(tt)/mround)
  pend  = ttmax*mround

  # loop over dictionaries with information
  for file in tdict:
    # determine number of samples to cut out at start
    tstart = tdict['%s' %(file)][0]
    nskip  = pstart-tstart
    # determine number of samples to remove at end
    tend   = tdict['%s' %(file)][1]
    ncut   = tend-pend
    # calculate number of samples
    nmin   = tend-tstart-nskip-ncut
    print tend,tstart,nskip,ncut

    # trim the edges (including the extra minute)
    wind_com = """Window3d < %s n1=30000 n2=%d f2=%d synch=1 > tmp.h""" %(file,nmin,nskip)
    print wind_com
    commands.getoutput(wind_com)

    # reshape the file
    nlen = 30000*nmin
    echo_com = """echo "n1=%d n2=1 o2=0 d2=1 hff=-1" >> tmp.h""" %(nlen)
    print echo_com
    commands.getoutput(echo_com)

    # patch the files for overlap
    filename = file.split('/')[-1]
    npatch = 2*(float(nmin)/mpatch)-1
    nwind  = float(patchsec)/d1
    patch_com = """Patch < tmp.h npatch1=%d nwind1=%d > %s_%s""" %(npatch,nwind,prefix,filename)
    print patch_com
    commands.getoutput(patch_com)
    echo_com = """echo "o2=%d d2=1" >> %s_%s""" %(ttmin,prefix,filename)
    print echo_com
    commands.getoutput(echo_com)

    # create the master header using pad and more
    # note: header is just repeated; based on first entry
    wind_com = """Window < tmp.h@@ n2=1 > tmp2.h@@"""
    print wind_com
    commands.getoutput(wind_com)
    pad_com = """Pad < tmp2.h@@ n2out=%d extend=1 > %s_%s@@""" %(npatch,prefix,filename)
    print pad_com
    commands.getoutput(pad_com)
    echo_com = """echo "hff=%s_%s@@" >> %s_%s""" %(prefix,filename,prefix,filename)
    print echo_com
    commands.getoutput(echo_com)
    more_com = """more ./Par/forties-hdr.fmt >> %s_%s@@""" %(prefix,filename)
    print more_com
    commands.getoutput(more_com)
    echo_com = """echo "o2=%d d2=1" >> %s_%s@@""" %(ttmin,prefix,filename)
    print echo_com
    commands.getoutput(echo_com)

    
#------------------------------- 
# run commands
#-------------------------------
file,patchsec,prefix = parse_commandline()
tdict,pdict = grab_time_stamps(file)
pad_patch(tdict,pdict,patchsec,prefix)
