module read_picks_mod
  ! Module to read picks output from Sep_cube
  !
  ! Jason Chang
  ! January 2016

  !=========================================
  ! includes
  !=========================================
  implicit none


  !=========================================
  ! declare module variables
  !=========================================

  contains


  !=========================================
  ! read picks subroutine
  !    note: files can only contain numbers
  !=========================================
  subroutine read_picks(filename,picks)
    ! declare variables
    integer                                     :: ii,jj
    integer                                     :: iostatus,pos1,pos2
    integer                                     :: nrows,ncols
    integer,parameter                           :: iunit=13
    real,allocatable,dimension(:,:),intent(out) :: picks 
    character(len=*),parameter                  :: delimit=' ' 
    character(len=*),intent(in)                 :: filename
    character(len=256)                          :: buffer

    ! open file
    open(file=filename,unit=iunit,form='formatted',status='old',action='read',iostat=iostatus)
    if (iostatus.ne.0) then
      write(0,*) 'Cannot read input pick file'
      return
    else
      write(0,*) 'Opening pick file'
    end if

    ! set counters
    nrows = 0

    ! first count number of lines
    do while (iostatus.eq.0)
      ! read in lines (A stands for character)
      read(unit=iunit,fmt='(A)',iostat=iostatus) buffer

      ! set counters
      pos1  = 1
      pos2  = 0

      ! count dimensions (assumption is that file is regular)
      if (iostatus.eq.0) then
        ! update counters
        nrows = nrows+1
        ncols = 0

        ! update number of columns
        do while (pos2.ne.1)
          pos2 = index(buffer(pos1:),delimit)
          if (pos2.ne.1) then
            ncols = ncols+1
            pos1  = pos1+pos2
          end if
        end do
      end if
    end do

    ! now read in data knowing dimensions
    rewind(unit=iunit)
    allocate(picks(nrows,ncols))

    ! read in data again
    do ii=1,nrows
      read(unit=iunit,fmt='(A)') buffer
      read(buffer,*) picks(ii,:)
    end do

    close(unit=iunit)
    write(0,*) 'Finished reading pick file'

  end subroutine

end module
