module xcorrelation_mod
  ! Module containing processes to:
  !   - cross-correlation
  !   - cross-coherence?
  !   - deconvolution?
  !
  ! Jason Chang
  ! October 2014

  !=================================================
  ! includes
  !=================================================
  use fourier_mod
  implicit none

  !=================================================
  ! declare module variables
  !=================================================
  integer,private                            :: n1
  real,private                               :: rscale
  complex,allocatable,dimension(:),private   :: in1,in2,out1,out2
  complex,allocatable,dimension(:)           :: cdata1,cdata2,coutput

  contains

  !=================================================
  ! cross-correlation intialization subroutine
  !=================================================
  subroutine xcorr_init(n1_in)
    ! declare variables
    integer          :: n1_in

    ! pass variables into module
    n1 = n1_in
    rscale = sqrt(float(n1_in))

    ! allocate
    allocate(in1(n1))
    allocate(in2(n1))
    allocate(out1(n1))
    allocate(out2(n1))
    allocate(cdata1(n1))
    allocate(cdata2(n1))
    allocate(coutput(n1))

    ! initialize fft
    call fft_init(n1)

  end subroutine


 
  !=================================================
  ! close cross-correlation subroutine
  !=================================================
  subroutine xcorr_close()
    ! deallocate
    deallocate(in1,in2,out1,out2,cdata1,cdata2,coutput)

    ! close fft
    call fft_close()

  end subroutine



  !=================================================
  ! cross-correlation subroutine
  !=================================================
  subroutine xcorr(data1,data2,output)
    ! declare variables
    integer                          :: ii
    real,allocatable,dimension(:)    :: data1,data2,output

    ! change inputs to complex
    do ii = 1,n1
      cdata1(ii) = cmplx(data1(ii),0.)
      cdata2(ii) = cmplx(data2(ii),0.)
    end do

    ! call fft on data
    call fft1d(.false.,1.,cdata1)
    call fft1d(.false.,1.,cdata2)

    ! compute cross-correlation
    do ii = 1,n1
      ! data1 is like receiver, data2 is like source
      coutput(ii) = cdata1(ii)*conjg(cdata2(ii))
    end do

    ! call ifft on data
    call fft1d(.true.,1.,coutput)

    ! perform fftshift
    call fftshift(.false.,coutput)

    ! take real part
    do ii = 1,n1
      output(ii) = rscale*real(coutput(ii))
    end do

  end subroutine


end module
